"""Exceptions for poolcontrolpy
"""


class HostError(Exception):
    """Raised when Pool Controller connection failed.

    Attributes
    ----------
    host : str
        Host name or IP address
    port : int
        Port number
    """

    def __init__(self, host: str, port: int):

        super().__init__()
        self.host = host
        self.port = port


class ResourceError(Exception):
    """Raised when API resource returns unexpected status"""

    def __init__(self, status: int, url: str):
        super().__init__()
        self.url = url
        self.status = status


class ResourceTypeError(Exception):
    """Raised when API resource returns unexpected type"""

    def __init__(self, url: str):
        super().__init__()
        self.url = url



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['poolcontrolpy']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0', 'asyncio>=3.4.3,<4.0.0']

setup_kwargs = {
    'name': 'poolcontrolpy',
    'version': '0.2.5',
    'description': 'Package for accessing nodejs-poolController',
    'long_description': '# poolcontrolpy\n\nPackage for accessing nodejs-poolController for use with a planned Home Assistant integration.\n\n## Installation\n\n```bash\n$ pip install poolcontrolpy\n```\n\n## Usage\n\nConfigure pool controller and confirm successful connection.\n\n```python\nimport asyncio\nimport aiohttp\n\nfrom poolcontrolpy.poolcontrolpy import Controller\n\n\nasync def checkconnection():\n\n    async with aiohttp.ClientSession() as client:\n        controller = Controller(client, "10.0.20.13", 4200)\n        return await controller.checkconnect()\n\nresp = asyncio.run(checkconnection())\nprint(resp)\n```\n\n*Note: This example uses \'asyncio.run\' so it cannot be called when another asyncio event loop is running in the same thread.*\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`poolcontrolpy` was created by Kevin Robinson. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`poolcontrolpy` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Kevin Robinson',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import logging
import json

import aiohttp

from .exceptions import HostError, ResourceError, ResourceTypeError

_LOGGER = logging.getLogger(__name__)

RES_CONF = "/config"

class Controller:
    """Controller class representing one nodejs-poolController
    """
    def __init__(self, session: aiohttp.ClientSession, host: str, port: int) -> None:
        """Initialize with connection parameters

        Parameters
        ----------
        session : aiohttp.ClientSession
            'aiohttp.ClientSession' to use for connection to controller
        host : str
            Hostname or IP address to controller
        port : int
            Port number to controller. Typically 4200
        """
        self.session = session
        self._rh = _RequestsHandler(session, host, port)
        self._config: dict = {}
        self._type: str
        self._model: str
        self._version: str

    @property
    def type(self):
        """Controller type"""
        return self._type

    @property
    def model(self):
        """Controller model"""
        return self._model

    @property
    def version(self):
        """Controller software version"""
        return self._version

    async def checkconnect(self) -> bool:
        """Check successful connection by inspecting returned controller version

        Returns
        -------
        bool
            True for successful connection
        """
        try:
            data = await self._rh.get(RES_CONF)
        except HostError:
            return False

        return bool(not data['appVersion'] == "")

    async def fetchconfig(self) -> bool:
        """Fetch the current controller configuration

        Returns
        -------
        bool
            True for successful operation
        """
        try:
            self._config = await self._rh.get(RES_CONF)
        except HostError:
            return {}
        else:
            self._type = self._config["controllerType"]
            self._model = self._config["equipment"]["model"]
            self._version = self._config["appVersion"]

        return bool(not self._config['appVersion'] == "")


class _RequestsHandler:
    """Handler class to manage http requests to the controller
    """
    def __init__(self, session: aiohttp.ClientSession, host: str, port):
        self.headers = {"Accept": "application/json"}
        self.scheme = "http"

        self.session = session
        self.host = host
        self.port = port

    async def get(self, resource: str) -> str:
        """Method to get resource from Pool Controller API

        Parameters
        ----------
        resource : str
            Resource to get with leading '/'

        Returns
        -------
        str
            String representing resource body

        Raises
        ------
        ResourceTypeError
            Error raised when ContentType is not application/json
        HostError
            Error raised when Host refuses connection
        ResourceError
            Error raised when Resource return status code >= 400
        """
        data = []
        url = f"{self.scheme}://{self.host}:{self.port}{resource}"

        try:
            async with self.session.get(
                url,
                headers=self.headers,
                raise_for_status=True
            ) as resp:
                try:
                    data = await resp.json()
                except aiohttp.ContentTypeError as error:
                    _LOGGER.debug(repr(error))
                    raise ResourceTypeError(error.request_info.url) from error
                else:
                    _LOGGER.debug(json.dumps(data))
                    return data
        except aiohttp.ClientConnectorError as error:
            _LOGGER.warning(
                "Connection to PoolController failed: %s://%s:%s",
                self.scheme,
                self.host,
                self.port)
            _LOGGER.debug(repr(error))
            raise HostError(error.host, error.port) from error
        except aiohttp.ClientResponseError as error:
            _LOGGER.warning("Received unexpected response for resource: %s", resource)
            _LOGGER.debug(repr(error))
            raise ResourceError(error.status, error.request_info.url) from error
        else:
            _LOGGER.debug(
                "Connected to PoolController: %s://%s:%s",
                self.scheme,
                self.host,
                self.port)
